@@ -6,6 +6,7 @@
  */
 
 #include "database.h"
+#include "databaselogging.h"
 
 #include <QCryptographicHash>
 #include <QDateTime>
@@ -13,12 +14,15 @@
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
+#include <QRandomGenerator>
 #include <QRegularExpression>
 #include <QSqlDatabase>
 #include <QSqlError>
 #include <QStandardPaths>
+#include <QThread>
 #include <QUrl>
 
+#include "error.h"
 #include "settingsmanager.h"
 
 #define TRUE_OR_RETURN(x)                                                                                                                                      \
@@ -76,6 +80,8 @@ bool Database::migrate()
         TRUE_OR_RETURN(migrateTo9());
     if (dbversion < 10)
         TRUE_OR_RETURN(migrateTo10());
+    if (dbversion < 11)
+        TRUE_OR_RETURN(migrateTo11());
     return true;
 }
 
@@ -180,6 +186,8 @@ bool Database::migrateTo8()
     qDebug() << "Migrating database to version 8; this can take a while";
 
     const int maxFilenameLength = 200;
+    static QRegularExpression asciiRegexp(QStringLiteral("[^a-zA-Z0-9 ._()-]"));
+
     QString enclosurePath = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
     if (!SettingsManager::self()->storagePath().isEmpty()) {
         enclosurePath = SettingsManager::self()->storagePath().toLocalFile();
@@ -196,7 +204,7 @@ bool Database::migrateTo8()
         QString name = query.value(QStringLiteral("name")).toString();
 
         // Generate directory name for enclosures based on feed name
-        QString dirBaseName = name.remove(QRegularExpression(QStringLiteral("[^a-zA-Z0-9 ._()-]"))).simplified().left(maxFilenameLength);
+        QString dirBaseName = name.remove(asciiRegexp).simplified().left(maxFilenameLength);
         dirBaseName = dirBaseName.isEmpty() ? QStringLiteral("Noname") : dirBaseName;
         QString dirName = dirBaseName;
 
@@ -223,7 +231,6 @@ bool Database::migrateTo8()
     TRUE_OR_RETURN(execute(query));
     while (query.next()) {
         QString queryTitle = query.value(QStringLiteral("title")).toString();
-        QString queryId = query.value(QStringLiteral("id")).toString();
         QString queryUrl = query.value(QStringLiteral("url")).toString();
         QString feedDirName = query.value(QStringLiteral("dirname")).toString();
 
@@ -232,7 +239,7 @@ bool Database::migrateTo8()
 
         if (QFileInfo::exists(legacyPath) && QFileInfo(legacyPath).isFile()) {
             // Generate filename based on episode name and url hash with feedname as subdirectory
-            QString enclosureFilenameBase = queryTitle.remove(QRegularExpression(QStringLiteral("[^a-zA-Z0-9 ._()-]"))).simplified().left(maxFilenameLength);
+            QString enclosureFilenameBase = queryTitle.remove(asciiRegexp).simplified().left(maxFilenameLength);
             enclosureFilenameBase = enclosureFilenameBase.isEmpty() ? QStringLiteral("Noname") : enclosureFilenameBase;
             enclosureFilenameBase += QStringLiteral(".")
                 + QString::fromStdString(QCryptographicHash::hash(queryUrl.toUtf8(), QCryptographicHash::Md5).toHex().toStdString()).left(6);
@@ -275,38 +282,72 @@ bool Database::migrateTo10()
     return true;
 }
 
-bool Database::execute(const QString &query, const QString &connectionName)
+bool Database::migrateTo11()
 {
-    QSqlQuery q(connectionName);
-    q.prepare(query);
+    qDebug() << "Migrating database to version 11";
+    TRUE_OR_RETURN(transaction());
+    TRUE_OR_RETURN(execute(QStringLiteral("ALTER TABLE Feeds ADD COLUMN sortType INTEGER DEFAULT 0;")));
+    TRUE_OR_RETURN(execute(QStringLiteral("PRAGMA user_version = 11;")));
+    TRUE_OR_RETURN(commit());
+    return true;
+}
+
+bool Database::execute(const QString &queryString)
+{
+    QSqlQuery q;
+    q.prepare(queryString);
     return execute(q);
 }
 
 bool Database::execute(QSqlQuery &query)
 {
-    // NOTE that this will execute the query on the database that was specified
-    // when the QSqlQuery was created.  There is no way to change that later on.
-    if (!query.exec()) {
-        qWarning() << "Failed to execute SQL Query";
-        qWarning() << query.lastQuery();
-        qWarning() << query.lastError();
-        return false;
+    bool state = executeThread(query);
+
+    if (!state) {
+        Q_EMIT Database::instance().error(Error::Type::Database, QString(), QString(), query.lastError().type(), query.lastQuery(), query.lastError().text());
     }
-    return true;
+
+    return state;
 }
 
-bool Database::transaction(const QString &connectionName)
+bool Database::transaction()
 {
-    // use IMMEDIATE transaction here to avoid deadlocks with writes happening
-    // in different threads
-    QSqlQuery query(QSqlDatabase::database(connectionName));
-    query.prepare(QStringLiteral("BEGIN IMMEDIATE TRANSACTION;"));
+    // use raw sqlite query to benefit from automatic retries on execute
+    QSqlQuery query;
+    query.prepare(QStringLiteral("BEGIN TRANSACTION;"));
     return execute(query);
 }
 
-bool Database::commit(const QString &connectionName)
+bool Database::commit()
 {
-    return QSqlDatabase::database(connectionName).commit();
+    // use raw sqlite query to benefit from automatic retries on execute
+    QSqlQuery query;
+    query.prepare(QStringLiteral("COMMIT TRANSACTION;"));
+    return execute(query);
+}
+
+bool Database::executeThread(QSqlQuery &query)
+{
+    int retries = 0;
+
+    // NOTE that this will execute the query on the database that was specified
+    // when the QSqlQuery was created.  There is no way to change that later on.
+    while (!query.exec()) {
+        // only retry if it failed due to the db being locked, see bug 500697
+        if (query.lastError().nativeErrorCode() == QStringLiteral("5") && retries < m_maxRetries) {
+            retries++;
+            qCDebug(kastsDatabase) << "Failed to execute SQL Query; retrying (attempt" << retries << " of" << m_maxRetries << ")";
+            qCDebug(kastsDatabase) << query.lastQuery();
+            qCDebug(kastsDatabase) << query.lastError();
+            QThread::usleep(retries * m_timeout + QRandomGenerator::global()->bounded(-m_timeoutRandomness, m_timeoutRandomness));
+        } else {
+            qCDebug(kastsDatabase) << "Failed to execute SQL Query";
+            qCDebug(kastsDatabase) << query.lastQuery();
+            qCDebug(kastsDatabase) << query.lastError();
+            return false;
+        }
+    }
+    return true;
 }
 
 int Database::version()
