@@ -27,8 +27,15 @@
 #endif
 
 #include <KAboutData>
+#ifdef HAVE_KDBUSADDONS
+#include <KDBusService>
+#endif
+#include <KIconTheme>
 #include <KLocalizedContext>
 #include <KLocalizedString>
+#ifdef HAVE_WINDOWSYSTEM
+#include <KWindowSystem>
+#endif
 
 #ifdef WITH_BREEZEICONS_LIB
 #include <BreezeIcons>
@@ -53,6 +60,18 @@ Q_DECL_EXPORT
 
 int main(int argc, char *argv[])
 {
+    KIconTheme::initTheme();
+
+    // Check if we need to force the interface to mobile or desktop, or stick
+    // with the built-in Kirigami setting
+    if (SettingsManager::self() && SettingsManager::self()->interfaceMode() != 2) {
+        if (SettingsManager::self()->interfaceMode() == 0) {
+            qunsetenv("QT_QUICK_CONTROLS_MOBILE");
+        } else if (SettingsManager::self()->interfaceMode() == 1) {
+            qputenv("QT_QUICK_CONTROLS_MOBILE", "1");
+        }
+    }
+
     if (QSysInfo::currentCpuArchitecture().contains(QStringLiteral("arm")) && qEnvironmentVariableIsEmpty("QT_ENABLE_GLYPH_CACHE_WORKAROUND")) {
         qputenv("QT_ENABLE_GLYPH_CACHE_WORKAROUND", "1");
     }
@@ -111,15 +130,38 @@ int main(int argc, char *argv[])
                      QStringLiteral(KASTS_VERSION_STRING),
                      i18n("Podcast Player"),
                      KAboutLicense::GPL,
-                     i18n("© 2020–2024 KDE Community"));
+                     i18n("© 2020–2025 KDE Community"));
     about.addAuthor(i18n("Tobias Fella"), QString(), QStringLiteral("tobias.fella@kde.org"), QStringLiteral("https://tobiasfella.de"));
     about.addAuthor(i18n("Bart De Vries"), QString(), QStringLiteral("bart@mogwai.be"));
     KAboutData::setApplicationData(about);
 
 #ifndef Q_OS_ANDROID
+    QApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("kasts")));
+
     KCrash::initialize();
 #endif
 
+#ifdef HAVE_KDBUSADDONS
+    KDBusService service(KDBusService::Unique);
+    QObject::connect(&service, &KDBusService::activateRequested, &engine, [&engine](const QStringList &arguments, const QString &workingDirectory) {
+        Q_UNUSED(arguments);
+        Q_UNUSED(workingDirectory);
+        const auto rootObjects = engine.rootObjects();
+        for (auto obj : rootObjects) {
+            if (auto window = qobject_cast<QQuickWindow *>(obj)) {
+                KWindowSystem::updateStartupId(window);
+                KWindowSystem::activateWindow(window);
+                if (!window->isVisible()) {
+                    window->show();
+                }
+                // TODO: parse arguments and pass on to app
+
+                return;
+            }
+        }
+    });
+#endif
+
     about.setupCommandLine(&parser);
     parser.process(app);
     QString feedURL = parser.value(addFeedOption);
@@ -129,8 +171,6 @@ int main(int argc, char *argv[])
     }
     about.processCommandLine(&parser);
 
-    qmlRegisterSingletonInstance("org.kde.kasts.settings", 1, 0, "SettingsManager", SettingsManager::self());
-
     if (!SettingsManager::self()->colorScheme().isEmpty()) {
         ColorSchemer::instance().apply(SettingsManager::self()->colorScheme());
     }
